# -*- coding: utf-8 -*-
from __future__ import annotations

from jsonbourne.protocols import (
    Dumpable,
    JsonInterface,
    JsonInterfaceProperty,
    JsonInterfaceProtocol,
)

__all__ = (
    "JsonInterface",
    "JsonInterfaceProperty",
    "JsonInterfaceProtocol",
    "Dumpable",
)
